@@ -50,8 +50,8 @@ Possible values are:
 	- FragmentType several atoms linked and represented by a text such as COOH (only in GChemPaint).
 	- BondType a bond between two (or more) atoms.
 	- MoleculeType a molecule.
-	- ChainType a chain of atoms (only in GChemPaint)
-	- CycleType a cycle (only in GChemPaint)
+	- ChainType a chain of atoms.
+	- CycleType a cycle.
 	- ReactantType a molecule involved in a reaction (only in GChemPaint).
 	- ReactionArrowType a reaction arrow (only in GChemPaint).
 	- ReactionOperatorType a + sign in a reaction (only in GChemPaint).
@@ -415,7 +415,7 @@ every derived class for which alignment has a meaning should implement this meth
 @param y y coordinate of the mouse click.
 
 This method is called to build a contextual menu for the object. It is called by Object::ShowContextualMenu, so
-it should not be necessary to call it directly. It should be overrided by derived classes when a contextual menu
+it should not be necessary to call it directly. It should be overriden by derived classes when a contextual menu
 is needed. Typically, each class adds a submenu and calls the same method for its parent.
 Default implementation calls registered BuildMenuCb callbacks and the parent's method.
 Derived classes should call Object::BuildContextualMenu before returning.
