@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * programs/gchemtable-curve.h 
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -48,14 +48,16 @@ public:
 	void OnClose ();
 	void OnProperties ();
 
-	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context);
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const;
 	void SetGraph (GogGraph *graph);
 	GtkWindow *GetGtkWindow () {return GTK_WINDOW (dialog);}
+	void ClearGuru () {m_Guru = NULL;}
+	void SaveAsImage (std::string const &filename, char const *mime_type, unsigned width, unsigned height) const;
 
 private:
 	std::string m_Name;
 	GogGraph *m_Graph;
-	GtkWidget *m_GraphWidget, *m_GraphBox;
+	GtkWidget *m_GraphWidget, *m_GraphBox, *m_Guru;
 };
 
 #endif	// GCHEMTABLE_CURVE_H
